// This is a C++ class for strings since Microsoft only provides one for Windows applications.

#ifndef CSTRING_H
#define CSTRING_H

#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

const MAX_LENGTH = 255;

class Cstring
{	private:
		char *str;					// Pointer to start of a C_style string
		int maxSize;				// Maximum length for str
	public:
		// Default constructor - constructs an empty string of MAX_LENGTH.
		// Postcond: An empty string of MAX_LENGTH has been constructed, maxSize is set as 255.
		Cstring();

		// Constructor - constructs an empty string of the specified length,len.
		// Precond:  length must be greater than zero.
		// Postcond: maxSize is length, and an empty string of the specified length has been 
		//			 constructed.
		Cstring(int len);

		// Copy constructor - constructs a string that has the same maxSize and value as fromStr
		// Postcond: self is a copy (both value and maxSize) of fromStr.
		Cstring(const Cstring & fromStr);

		// Converts a C-style string to a Cstring.
		// Precond:  fromStr must be a null terminated character array (i.e. a C style string)
		// Postcond: The value part of self is fromStr and maxSize is the length of fromStr
		Cstring(const char *fromStr);

		// Destructor releases dynamic storage for str
		//~Cstring();

		// Converts a Cstring object to a C style string, by returning the eqivalent of str
		char * c_str()const;

		// Determines the length of str.
		int length()const;

		// Looks for first occurrence of c in str, beginning at index start. Return the 
		// index of c in str if match occurs; otherwise return -1.
		// Precond: start if specified must be greater than or equal to 0
		int findFirstOf(char c, int start=0)const;

		// Looks for the last occurrence of c in str. Return the index of the match if it 
		// occurs; otherwise return -1.
		int findLastOf(char c)const;

		// Copy count characters from self beginning at index start, and return the resulting 
		// string as a Cstring object. If the tail of str has fewer than count characters or 
		// count is -1 the rest of the string is copied stopping at the end of the string.
		// Precond: start must be greater than or equal to 0. count must be greater than or 
		//			equal to -1
		Cstring substr(int start=0, int count=-1)const;

		// Locates a specific pattern in the string.  The method takes a Cstring object s and 
		// and a index start as arguments and looks for mathc of s as a substring in self. It
		// returns the starting index of the match if it occurs; otherwise it returns -1.
		int find(const Cstring &s, int start=0)const;

		// Locates a specific pattern in the string.  The method takes a C-style string  s and 
		// and a index start as arguments and looks for mathc of s as a substring in self. It
		// returns the starting index of the match if it occurs; otherwise it returns -1.
		int find(const char *s, int start=0)const;

		// Place the substring s into the string beginning at index start.  The insertion expands
		// the maxSize of the string if necessary.
		// Precond: start must be greater than or equal to 0 and less than or equal to the length 
		//			of str.
		// Postcond: The substring s has been placed into str at the specified position. The 
		//			 maxSize of self has been increased if necessary.
		void insert(int start, const Cstring &s);

		// Delete count characters from str, begining at index start. If fewer than count 
		// characters exist in the tail of the string or if count is -1, delete characters 
		// up to the end of the string.
		// Precond: start must be greater than or equal to 0
		// Postcond: The specified number of characters has been removed from str.
		void erase(int start=0, int count=-1);

		// Overloads operation +. Concatenates string s to self by copying s after self. 
		// s is left unchanged. maxSize of self is increased if necessary.
		Cstring & operator+(const Cstring & s);

		// Overloads operation +. Concatenates C-style string s to self by copying s after self. 
		// s is left unchanged. maxSize of self is increased if necessary.
		Cstring & operator+(const char * s);

		// Overloads operation +. Concatenates character c to self by copying c after self. 
		// c is left unchanged. maxSize of self is increased if necessary.
		Cstring & operator+(const char c);

		// Overloads operation +=. Concatenates string s to self by copying s after self. 
		// s is left unchanged. maxSize of self is increased if necessary.
		Cstring & operator +=(const Cstring & s);

		// Overloads operation <.Returns true if self is less than the parameter.
		bool operator < (const Cstring & s)const;

		// Overloads operation <=. Returns true if self is less than or equal to the parameter.
		bool operator <= (const Cstring & s)const;

		// Overloads operation >. Returns true if self is greater than the parameter.
		bool operator > (const Cstring & s)const;

		// Overloads operation >=. Returns true if self is greater than or equal to the parameter.
		bool operator >= (const Cstring & s)const;

		// Overloads operation ==. Returns true if self is equivalent the parameter.
		bool operator == (const Cstring & s)const;

		// Overloads operation !=. Returns true if self is not equivalent to the parameter.
		bool operator != (const Cstring & s)const;

		// Overloads the assignment operator, returning a pointer to the object self.
		Cstring & operator = (const Cstring &s);

		// Overloads operation []. Returns the character specified by the index of str
		char &operator [] (int i);



		//friend functions to overload I/O operators and methods
		friend istream & operator >> (istream& istr, Cstring &s);

		friend istream & getline(istream &istr, Cstring &s);

		friend ostream & operator << (ostream& ostr, Cstring &s);



};


#endif